#
# Evantide/State.pm
#
#  persistent state routines
#  evantide, ltd. staff (staff@evantide.com)
#
# TODO: Destructor should store the state; or should it?
#

package Evantide::State;

use Exporter;
use FileHandle;
use Storable qw(store retrieve);

@ISA       = qw(Exporter);
@EXPORT    = qw(new put get remove get_oid sync);

$StateDir = "/var/tmp/state";

sub new
{
    my $type = shift;
    my ($self, $oid, $hashref);

    if ($oid = shift) {
	return undef if not
	    $hashref = Storable::retrieve ("$StateDir/OS$oid");
    }
    else {
	my $fd = 0;

	do {
	    $oid = a_random_number ();
	} while (not sysopen ($fd, "$StateDir/OS$oid", O_CREAT | O_EXCL,0600));
	
	close ($fd);
    }

    $self->{oid}  = $oid;
    $self->{file} = "$StateDir/OS$oid";
    $self->{hash} = $hashref ? $hashref : {};

    bless $self, $type;
    $self->sync ();

    return $self;
}


sub sync
{
    my $self = shift;

    Storable::store ($self->{hash}, $self->{file});
}


sub put
{
    my ($self, $name, $hashref) = @_;

    $self->{hash}{$name} = $hashref;
}

sub get
{
    my ($self, $name) = @_;

    return $self->{hash}{$name};
}

sub remove
{
    my ($self, $name) = @_;

    delete $self->{hash}{$name};
}

sub get_oid
{
    return ($_[0]->{oid});
}


#
# a_random_number ()
#
#   Returns an integer from a cryptographically strong entropy pool.
#
sub a_random_number
{
    my ($randomfd, $random);
    $randomfd = 0;

    open  ($randomfd, "/dev/urandom");
    read  ($randomfd, $random, 4);
    close ($randomfd);
    return (unpack ("L", $random));
}
